
var Scat     = Scat || {};
var BASE_URL = '/opmon/seagull/www/index.php/%s/action/';
Scat = {

  HOST: 1,

  SERVICE_CATALOG: 2,

  SERVICE: 3,
	
	/**
	 * Selected Scat id
	 * 
	 * @type int
	 */
	_id: null,             
	
	/**
	 * Contact list 
	 * 
	 * @type Object
	 */
	_contacts: {},
	
	/**
	 * Contactgroup list 
	 * 
	 * @type Object
	 */
	_contactGroups: {},
	
	/**
	 * Scat list
	 * 
	 * @type Array
	 */
	_scatList: new Array(),

		/**
	 * Host list
	 * 
	 * @type Array
	 */
	_hostList: new Array(),

		/**
	 * Service list
	 * 
	 * @type Array
	 */
	_serviceList: new Array(),

	/**
	 * Scat content
	 * 
	 * @type Array
	 */
	_scatContent   : new Array(),
	
	/**
	 * Perform HTTP request via ajax or fetch
	 * 
	 * @param string method
	 * @param string url
	 * @param object data
	 * @param function cb
	 * @returns void
	 */
  _makeRequest: ( method, url, data, mask, cb=((err, x) => [err, x]) ) => {
		
		// Add ajax spinner
		if (mask) {
			mask = Scat._getAjaxSpinner(mask).insertBefore(mask);
		} else {
			mask = $();
		}

		// Set pre-defined post variables (Is it really necessary?)
		if ( method === 'POST' ) {
			data.mod                 = 'SCAT';
			data.hostServiceProtocol = 'https';
		}
		
		if (typeof fetch === 'function') { // Fetch API
			
			return fetch( BASE_URL.replace('%s','wsconnector') + url,
			{
				cache:   "no-store",
				method:  method,
				body:    $.param(data),
				headers: {
					"content-type":"application/x-www-form-urlencoded"
				}
			})
			.finally( () => mask.remove() )
      .then(response => response.json())
      .then(json => cb(null,json), cb)
			
		} else { // Ajax fallback
			
			return $.ajax({
				url:    BASE_URL.replace('%s','scat') + url,
				method: method,
				data:   data
			})
			.always(()           => mask.remove())
			.done( (data)        => cb(null,data))
			.fail( (jqXHR, data) => Scat._handleErrors(data));
	
		}
		
	},
	
	/**
	 * Create ajax spinner
	 * 
	 * @param selector mask
	 * @returns jQuery object
	 */
	_getAjaxSpinner: (mask) => {
		return $(
				'<div class="spinner-border spinner-border-sm" role="status" style="position: absolute; bottom: 12px; left: ' + $(mask).width() + 'px;">' + 
				'<span class="sr-only">Loading...</span>' + 
				'</div>'
		);
	},
	
	/**
	 * Handle request errors
	 * 
	 * @param mixed error
	 * @returns void
	 */
	_handleErrors: (error) => {
		console.error(error);
	},
	
	/**
	 * Dynamically populate dropdown
	 * 
	 * @param string selector
	 * @param array  array
	 * @param string prop
	 * @param boolean clear
	 * @returns void
	 */
	_populateDropdown: (selector,array,prop,clear) => {
		
		// Clear dropdown
		if (clear)
			$(selector).children().remove();

		// Populate with data
		$.each( array, item => {
			$(selector).append($('<option />').val(item).text( (prop) ? array[item][prop] : array[item]));
		});
		
	},
	
	/**
	 * Populate user table
	 * 
	 * @returns void
	 */
	_populateUserTable: (arr) => {
		$.each(['scatUsers','scatKeyUsers','scatBusinessAccountable','scatTechnicalAccountable'],
			(i,v) => {
				$.each(arr, (j,o) => $('#' + v).DataTable().row.add([j,j,o.alias]));
				$('#' + v).DataTable().draw();
			}
		);
	},

	/**
	 * Helper function to apply service catalog content
	 * 
	 * @returns void
	 */
	_applyScatContent: () => {
		// Remove undefined values
		Scat._scatContent = Scat._scatContent.filter( x => { return x !== undefined } );
		
		// Update row values and re-draw
		$('#scatContent')
			.DataTable()
			.clear()
			.rows.add( Scat._scatContent )
			.draw();
	},
	
	/**
	 * Helper function to add service catalog content
	 * 
	 * @param Array value
	 * @returns void
	 */
	_addScatContent: (value) => {
		var found = 0;
		$.each( Scat._scatContent, (i,a) => {
			if (value[0] === a[0])
				found = 1;
		});
		if (!found)
			Scat._scatContent.push(value);
	},

	/**
	 * Get object preffix to be used on scat content
	 * 
	 * @returns string
	 */
	_getObjectPreffix: () => {
		
		var id = $('#scatObjectType').children('option:selected').val();
    var types = ["Host", "Serivce Catalog", "Serivce"]
    var getType = Scat._getProp([Number(id) - 1], "")
    return getType(types);
	},
	
	/**
	 * Display alert
	 * 
	 * @param string msg
	 * @returns void
	 */
	_showAlert: (msg) => {
		$('#scatModal').modal('hide');
		$('#scatMsgBody').html(msg);
		$('#scatAlert').modal();
	},

	/**
	 * Select grid contacts based on given data
	 * 
	 * @param Array data
	 * @param string selector
	 * @returns void
	 */
	_selectContacts: ( data, selector ) => {
		
		if (!$.isArray(data)) return;
		$.each(data, (i,v) => {
			var id = (v.type == '10' ? 'g' : 'c') + '_' + v.value;
			$.each( $(selector).DataTable().data(), ( idx, row ) => {
				if ( row[0] == id )
					$(selector).DataTable().row( idx ).select();
			});
		});
		
	},

  /**
   * Get prop from an element with default return
   *
   * (Array String, a) -> b -> a | c
   */
  _getProp: (keys=[], defVal) => obj => (
    keys.reduce((meta, key) => (
      meta.done
        ? meta
        : !meta.acc.hasOwnProperty(key)
            ? { ...meta, value: defVal, done: true }
            : { ...meta, acc: obj[key], value: obj[key] }
    ), { acc: obj, value: null, done: false })
    .value
  ),

	/**
	 * Init module
	 * 
	 * @returns void
	 */
	init: () => {

    if (parent && parent.fullScreenWithoutCookie) {
      parent.fullScreenWithoutCookie();
    }
		
		$.fn.dataTable.ext.errMode = 'none';
		
		// Initialize datepicker
		$( '#scatNextRevision' ).datepicker({ dateFormat: 'dd-mm-yy', minDate: new Date() });
		
		// Initialize scatSlaYear
		var yArr = {};
		for( var i = new Date().getFullYear(); i>= 2014; i-- )
			yArr[i] = i;
		Scat._populateDropdown('#scatSlaYear',yArr,false,true);
		$('#scatSlaYear').val(new Date().getFullYear());

    // Initialize data table
    Scat.initializeDataTable();
	
		// Initialize user tables
		Scat.initScatUserTables();
		
		// Initialize service catalog content table
		Scat.initScatContent();
		
		// Initialize SLA cell editing
		Scat.initSlaEditing();
		
		// Initialize Custom Attribute grid
		Scat.initGridCustomAttribute();
		
		// Get all service catalogs
		Scat.getServiceCatalogs()
      .then(() => (
				Scat._populateDropdown('#scats',Scat._scatList,'name',false)
      ));
		
		// Get scat types
		Scat.getScatTypes();
		
		// Get timecuts
		Scat.getTimeCuts();
		
		// Get hosts
    Scat.getHostsByServices()
      .then(() => (
        Scat.populateDataTable(Scat.getTransformer(Scat.HOST).toDataTable())
      ));
		
		// Get contacts and groups
		Scat.getContacts();

    // get all services
    Scat.getServices();
		
	},
	
	/**
	 * Load saved service catalogs
	 * 
	 * @returns void
	 */
	getServiceCatalogs: () => (
		Scat._makeRequest('POST','getservicecatalogs',{},'#scats')
      .then(([ err, data ]) => {
        Scat._scatList = data
        return data
      })
  ),
	
	/**
	 * Load scat types
	 * 
	 * @returns void
	 */
	getScatTypes: () => {
	
		Scat._makeRequest('POST','getscattypes',{},'#scatType',
			(err,data) => {
				Scat._populateDropdown('#scatType',data,false,true);
			}
		);
		
	},
	
	/**
	 * Load timecuts
	 * 
	 * @returns void
	 */
	getTimeCuts: () => {
	
		Scat._makeRequest('POST','gettimecuts',{},'#scatTimeCut',
			(err,data) => {
				Scat._populateDropdown('#scatTimeCut',data,false,true);
				Scat._populateDropdown('#scatSlaTimeCut',data,false,true);
			}
		);
		
	},
	
	/**
	 * Get hosts
	 * 
	 * @returns void
	 */
	getHostsByServices: () => (
    Scat._makeRequest('POST','gethostsbyservices',{},'#scatObjectList')
      .then(([ err, data ]) => {
        Scat._hostList = data
        return data
      })
  ),
	
	/**
	 * Get services by selected host
	 * 
	 * @returns void
	 */
	getServices: () => (
    Scat._makeRequest('POST','getallservices',{},'#scatObjectList')
      .then(([ err, data ]) => {
        Scat._serviceList = data
        return data
      })
  ),
	
	/**
	 * Load contacts and contactgroups
	 * 
	 * @returns void
	 */
	getContacts: () => {

		// Load contacts
		Scat._makeRequest('POST','getcontacts',{},'#scatResponsable',
			(err,data) => {
				$.each(data, (i,o) => Scat._contacts["c_" + o.contact_id] = o );
				Scat._populateDropdown('#scatResponsable',Scat._contacts,'alias',false);
				Scat._populateUserTable(Scat._contacts);
			}
		);

		// Load contact groups
		Scat._makeRequest('POST','getcontactgroups',{},'#scatResponsable',
			(err,data) => {
				$.each(data, (i,o) => Scat._contactGroups["g_" + o.contactgroup_id] = o );
				Scat._populateDropdown('#scatResponsable',Scat._contactGroups,'alias',false);
				Scat._populateUserTable(Scat._contactGroups);
			}
		);
		
	},
	
	/**
	 * Initialize service catalog content
	 * 
	 * @returns void
	 */
	initScatContent: () => {
		
		$('#scatContent').dataTable({
			paging:    false,
			searching: false,
			info:      false,
			//ordering:  false,
			select:    true,
			//rowReorder:true,
			data:      [],
			order:     [[1,'asc']],
			scrollY:   200,
			language:  { emptyTable: "No object selected!" },
			columns: [
				{ visible: false }, // ID
				{ title: '#', width: 20,      render: (data,type,row,meta) => { return meta.row + 1; }}, // row number
				{ title: 'type', width: 80,      render: (data,type,row,meta) => { return data; }}, // row number
				{ title: "Object", render: (data,type,row,meta) => { // Object label
						return ($.isArray(data) ? data.join("<br>") : data);
				}}, 
				{ visible: false }, // Object original label
			],
			"drawCallback": () => {
				// Enable drag'n drop
				$( $('#scatContent').DataTable().table().body() ).sortable({
					start:  (event,ui) => {
						$(this).attr('data-previndex', ui.item.index());
					},
					update: (event,ui) => {
						var newPos = ui.item.index();
						var oldPos = $(this).attr('data-previndex');
						$(this).removeAttr('data-previndex');
						
						var aux = Scat._scatContent[oldPos];
						delete Scat._scatContent[oldPos];
						Scat._scatContent.splice(newPos,0,aux);
						Scat._applyScatContent();
					}
				});
				// Enable double click
				$('#scatContent').unbind('dblclick').on('dblclick','tr',(e) => {
					e.stopPropagation();
					var row = $( e.target ).parent();
					$('#scatContent').DataTable().row( row ).select();
					Scat.removeItem();
				});
			}
		});
		
	},
	
	/**
	 * Initialize user tables
	 * 
	 * @returns void
	 */
	initScatUserTables: () => {
		
		$.each(['scatUsers','scatKeyUsers','scatBusinessAccountable','scatTechnicalAccountable'],
			(i,v) => {
				$('#' + v).dataTable({
					paging:    false,
					searching: false,
					info:      false,
					select:    true,
					data:      [],
					order:     [[1,'asc'],[2,'asc']],
					language:  { emptyTable: "No users available!" },
					columns: [
						{ visible: false },
						{ render: (data,type,row,meta) => {
								return (data.substring(0,1) == 'c') ? 
									'<center><i class="far fa-user fa-lg"></i></center>' : 
									'<center><i class="fas fa-users fa-lg"></i></center>';
											
							}
						},
						{ title: "Users" }
					]
				});
			}
		);
		
	},
	
	/**
	 * Register event to edit SLA cell
	 * 
	 * @returns void
	 */
	initSlaEditing: () => {
		
		$('.sla-month').on('click',(e) => {
			var cell  = $(e.target);
			var width = cell.width();
			var value = cell.html();
			var input = $('<input>').attr({
				value:     value,
				type:      'text',
				'class':   'col-12 form-control form-control-sm'
			}).on('blur',(e) => {
				var el = $(e.target);
				if (el.val() < 0 || el.val() > 100) {
					el.addClass('is-invalid');
					$("<div class='invalid-feedback'>Invalid value</div>").insertAfter(el);
				} else {
					var td = el.parent();
					td.html( el.val() );
				}
			});
			cell.html(input).width(width);
			input.focus();
		});
		
	},
	
	/**
	 * Initialize Custom Attribute grid
	 * 
	 * @returns void
	 */
	initGridCustomAttribute: () => {

		$('#scatCustomAttributes').dataTable({
			paging:    false,
			searching: false,
			info:      false,
			data:      [],
			order:     [[0,'asc'],[1,'asc']],
			language:  { emptyTable: "No custom attributes defined!" },
			columns: [
				// Attribute name
				{ title: "Attribute Name", render: (data,type,row,meta) => {
						return $('<input>').attr({
							value:     data || '',
							type:      'text',
							'class':   'custom-attr col-12 form-control form-control-sm'
						}).wrap('<div></div>').parent().html();
					}
				},
				// Value
				{ title: "Value", render: (data,type,row,meta) => {
						return $('<input>').attr({
							value:     data || '',
							type:      'text',
							'class':   'custom-value col-12 form-control form-control-sm'
						}).wrap('<div></div>').parent().html();
					}
				},
				// Remove button
				{ title: "", render: (data,type,row,meta) => {
						return $('<center>'                                +
								 '<a class="btn btn-light btn-sm" onclick="Scat.removeCustomAttribute(this);">'        +
								 '<i class="fas fa-times fa-lg">&nbsp;</i>' +
								 'Remover'                                 +
								 '</a>'                                    +
								 '</center>'
						).wrap('<div></div>').parent().html();
					}
				},
			]
		});

	},
	
	/**
	 * Add custom attribute to grid
	 * 
	 * @returns void
	 */
	addCustomAttribute: () => {
		$('#scatCustomAttributes').DataTable().row.add([null,null,null]).draw();
	},
	
	/**
	 * Remove custom attribute from grid
	 * 
	 * @returns void
	 */
	removeCustomAttribute: (self) => {
		$('#scatCustomAttributes').DataTable().row( self.parentNode.parentNode.parentNode ).remove().draw();
	},
	
	/**
	 * Populate SLA Grid with given values
	 * 
	 * @returns void
	 */
	populateSlaGrid: () => {
		
		var value = $('#scatSlaValue').val();
		var warn  = $('#scatSlaWarning').val();
		
		if (value && warn) {
			$('.sla-month').slice(0,13).each( (idx,el) => {
				$(el).html( (idx == 12) ? warn : value );
			});
		}
		
	},
	
	/**
	 * Clear SLA grid
	 * 
	 * @return void
	 */
	clearSlaGrid: () => {
			$('.sla-month').slice(0,13).each( (idx,el) => {
				$(el).html(0);
			});
	},
	
	/**
	 * Modifica a exibição dos tipos de objeto
	 * 
	 * @returns void
	 */
	changeObjectType: () => {
		var id = $('#scatObjectType').children('option:selected').val();
    var arr = Scat.getTransformer(id).toDataTable();
    Scat.populateDataTable(arr);
	},

  transformElements: () => ({
    [Scat.HOST]: {
      toDataTable: () => (
        Object.keys(Scat._hostList).map(key => ([ Scat._hostList[key], key ]))
      )
    },
    [Scat.SERVICE_CATALOG]: {
      toDataTable: () => (
        Object.values(Scat._scatList).map(scat => ([ scat.id, scat.name ]))
      )
    },
    [Scat.SERVICE]: {
      toDataTable: () => (
        Object.values(Scat._serviceList).map(service => ([
          service.host_id + "!" + service.service_id,
          `<span class="host">${service.host_name}</span> <span class="service">${service.service_description}</span>`
        ]))
      )
    },
    empty: {
      toDataTable: () => ([])
    }
  }),

  getTransformer: id => (
    Scat._getProp([id], Scat.transformElements().empty)(Scat.transformElements())
  ),
	
	/**
	 * Add all items to scat content
	 * 
	 * @returns void
	 */
	addAllItems: () => {
		$.each( $('#scatObjectContent').DataTable().data(), (i,a) => {
			Scat._addScatContent([ a[0], a[0], Scat._getObjectPreffix(), a[1] ]);
		});
		$('#scatObjectContent').DataTable().clear().draw();
		Scat._applyScatContent();
	},
	
	/**
	 * Add selected items to scat content
	 * 
	 * @returns void
	 */
	addItem: () => {
		$.each($('#scatObjectContent').DataTable().rows( { selected: true } ), (i,v) => {
			if (!v.length)
				return;
			v.forEach( (x) => {
				var sel = $('#scatObjectContent').DataTable().row(x).data();
				Scat._addScatContent([ sel[0], sel[0], Scat._getObjectPreffix(), sel[1] ]);
			});
			v.forEach( (x,idx) => {
				$('#scatObjectContent').DataTable().row(x - idx).remove().draw();
			});
			Scat._applyScatContent();
		});
	},
	
	/**
	 * Remove selected items from scat content
	 * 
	 * @returns void
	 */
	removeItem: () => {
		$.each($('#scatContent').DataTable().rows( { selected: true } ), (i,v) => {
			if (!v.length)
				return;
			v.forEach( (x, idx) => {
				var sCont = Scat._scatContent[x];
        if ($("#scatObjectType option:selected").text() === sCont[2]) {
          $('#scatObjectContent').DataTable().row.add([sCont[1],sCont[3]]).draw();
        }
				Scat._scatContent = Scat._scatContent.filter( (el,i) => { return (i === (x-idx)) ? false : true; });
			});
		});
		Scat._applyScatContent();
	},
	
	/**
	 * Remove all items from scat content
	 * 
	 * @returns void
	 */
	removeAllItems: () => {
		var id = $('#scatObjectType').children('option:selected').val();
		if (id == 1) {
      Scat.populateDataTable(Scat.getTransformer(Scat.HOST).toDataTable())
		} else {
			Scat.changeObjectType();
		}
		Scat._scatContent = new Array();
		Scat._applyScatContent();
	},
	
	/**
	 * Parallelize selected items on scat content list
	 * 
	 * @returns void
	 */
	parallelizeItems: () => {
		$.each($('#scatContent').DataTable().rows( { selected: true } ), (i,v) => {
			if (!v.length)
				return;
			var arr = [ [], [], [], [] ];
			var firstPos = null;
			v.forEach( (x) => {
				var sCont = Scat._scatContent[x];
				if (firstPos == null)
					firstPos = x;
				if ($.isArray(sCont[0])) {
					$.each(sCont[0],(idx,el) => {
						arr[0].push( sCont[0][idx] );
						arr[1].push( sCont[1][idx] );
						arr[2].push( sCont[2][idx] );
						arr[3].push( sCont[3][idx] );
					});
				} else {
					arr[0].push( sCont[0] );
					arr[1].push( sCont[1] );
					arr[2].push( sCont[2] );
					arr[3].push( sCont[3] );
				}
				delete( Scat._scatContent[x] );
			});
			Scat._scatContent.splice(firstPos,0,arr);
			//Scat._scatContent.push(arr);
		});
		Scat._applyScatContent();
	},
	
	/**
	 * Serialize selected items
	 * 
	 * @returns void
	 */
	serializeItems: () => {
		$.each($('#scatContent').DataTable().rows( { selected: true } ), (i,v) => {
			if (!v.length)
				return;
			v.forEach( (x) => {
				var sCont = Scat._scatContent[x];
				if ($.isArray(sCont[0])) {
					$.each(sCont[0],(idx,el) => {
						var arr = [ sCont[0][idx], sCont[1][idx], sCont[2][idx], sCont[3][idx] ];
						Scat._scatContent.push(arr);
					});
				} else {
					Scat._scatContent.push(sCont);
				}
				delete( Scat._scatContent[x] );
			});
		});
		Scat._applyScatContent();
	},
	
	/**
	 * Get selected contacts by grid
	 * 
	 * @param string selector
	 * @returns Array
	 */
	getContactsByGrid: (selector) => {
		
		var arr = new Array();
		$.each($(selector).DataTable().rows( { selected: true } ), (i,v) => {
			v.forEach( idx => {
				var data = $(selector).DataTable().row(idx).data();
				arr.push({
					alias: data[2],
					type:  (data[0].split("_")[0] === 'c' ? 9 : 10),
					value: data[0].split("_")[1]
				});
			});
		});
		return arr;
		
	},
	
	/**
	 * Detect changes on Scat name to display rename/save as option
	 * 
	 * @returns void
	 */
	detectScatNameChange: () => {
		if (Scat._id === null) return;
		
		var value        = $('#scatName').val();
		var selectedScat = $('#scats').children('option:selected').text();
		if (value != selectedScat) {
			$('.btn-save').html('Save/Rename');
			$('.btn-save-as').show();
		} else {
			$('.btn-save').html('Save');
			$('.btn-save-as').hide();
		}
	},
	
	/**
	 * Rename given Service Catalog
	 * 
	 * @returns void
	 */
	saveAs: () => {
		Scat._id = null;
		Scat.save();
	},
	
	/**
	 * Save Service Catalog
	 * 
	 * @returns void
	 */
	save: () => {
	
		if ($('#scatName').val() === '')
			return;
	
		if (!Scat._scatContent.length)
			return Scat._showAlert('Please select the services that belongs to this Service Catalog');
		
		// Populate Seasonality
		var sazDays = new Array();
		$("[id*='sazDay']").each( (i,v) => {
			sazDays.push(($('#' + v.id).is(':checked') ? 1 : 0));
		});
		
		// Populate levels
		var levels = new Array();
		$.each( Scat._scatContent, (i,v) => {
			levels.push({
				lvl:   i+1,
				label: ($.isArray(v[2]) ? v[2].join("\n") : v[2]),
				value: ($.isArray(v[1]) ? v[1].join("\n") : v[1]),
				host:  ($.isArray(v[2]) ? v[2].join("\n") : v[2])
			});
		});
		
		// Populate custom attr
		var customAttr = new Array();
		$('.custom-attr').each( (i,v) => {
			customAttr.push({
				global:   "false",
				cf_name:  v.value,
				nid:      i+1,
				"var":    "[custom]",
				cf_id:    null
			});
		});
		$('.custom-value').each( (i,v) => {
			customAttr[i].cf_value = v.value;
		});
		
		// Create JSON Data
		var jsonData = {
			name:                         $('#scatName').val(),
			type:                         $('#scatType').children('option:selected').val(),
			responsable:                  [{
					alias: $('#scatResponsable').children('option:selected').text(),
					type:  ($('#scatResponsable').children('option:selected').val().split("_")[0] === 'c' ? 9 : 10),
					value: $('#scatResponsable').children('option:selected').val().split("_")[1]
			}],
			cost:                         $('#scatCost').val(),
			nextRevision:                 $('#scatNextRevision').val(),
			levels:                       levels,
			selectedTimecut:              $('#scatTimeCut').children('option:selected').val(),
			resolutionTime:               $('#scatResolutionTime').val(),
			answerTime:                   $('#scatTreatmentTime').val(),
			selectedSeverity:             $('#scatSeverity').children('option:selected').val(),
			imageInjury:                  $('#scatFinancialLoss').children('option:selected').val(),
			complexity:                   $('#scatComplexity').children('option:selected').val(),
			vipUsers:                     $('#scatVIPUsers').children('option:selected').val(),
			selectedAffectedUsers:        $('#scatAffectedUsers').children('option:selected').val(),
			dependencyRelationship:       $('#scatDependencyRelationship').children('option:selected').val(),
			sazDays:                      sazDays,
			description:                  $('#scatDescription').val(),
			dependentBusinessProcesses:   $('#scatDependentBusinessProcesses').val(),
			serviceDeliverable:           $('#scatServiceDeliverable').val(),
			txtUser:                      $('#scatTxtUsers').val(),
			selectedUser:                 Scat.getContactsByGrid('#scatUsers'),
			txtKeyUser:                   $('#scatTxtKeyUsers').val(),
			selectedKeyUser:              Scat.getContactsByGrid('#scatKeyUsers'),
			txtBusinessAccountable:       $('#scatTxtBusinessAccountable').val(),
			selectedBusinessAccountable:  Scat.getContactsByGrid('#scatBusinessAccountable'),
			txtTechnicalAccountable:      $('#scatTxtTechnicalAccountable').val(),
			selectedTechnicalAccountable: Scat.getContactsByGrid('#scatTechnicalAccountable'),
			involvedApplications:         $('#scatInvolvedApps').val(),
			custom_fields:                customAttr,
			activities:                   [],
			deleteActivities:             []
		};
		
		// Service Catalog id
		if (Scat._id !== null)
			jsonData.id = Scat._id;
		
		// Envia os dados para o WS
		Scat._makeRequest('POST','savescat', {  jsonData: JSON.stringify(jsonData) },'#scatName',
			(err,data) => {
			
				// Error handling
				if (err || data.error)
					return Scat._showAlert('Error saving Service Catalog component');
			
				// Extract ID and name
				var id   = Object.keys(data).shift();
				var name = Object.values(data).shift();
				Scat._id = id;

				// Save SLA
				Scat.saveSla(id,name);
		
				// Add new id to scat dropdowon
				$('#scats').append($('<option />').val(id).text(name)).val(id);

		});
	},
	
	/**
	 * Save Sla for a given Service Catalog
	 * 
	 * @param int id
	 * @param string name
	 * @returns void
	 */
	saveSla: (id,name,cb) => {
		
		// Create JSON Data
		var jsonData = {
			formTo:   4,
			year:     $('#scatSlaYear').children('option:selected').text(),
			timecut:  $('#scatSlaTimeCut').val(),
			host_id:  id,
			services: [{
				id:      id,
				label:   "<b>" + name + "</b>",
				name:    name,
				image:   null,
				iskey:   0,
				origin:  0,
				warning: $('.sla-warning').html(),
				jan:     $('.sla-month:eq(0)').html(),
				feb:     $('.sla-month:eq(1)').html(),
				mar:     $('.sla-month:eq(2)').html(),
				apr:     $('.sla-month:eq(3)').html(),
				may:     $('.sla-month:eq(4)').html(),
				jun:     $('.sla-month:eq(5)').html(),
				jul:     $('.sla-month:eq(6)').html(),
				aug:     $('.sla-month:eq(7)').html(),
				sep:     $('.sla-month:eq(8)').html(),
				oct:     $('.sla-month:eq(9)').html(),
				nov:     $('.sla-month:eq(10)').html(),
				dec:     $('.sla-month:eq(11)').html()
			}]
		};
		
		// Envia os dados para o WS
		Scat._makeRequest('POST','savesla', {  jsonData: JSON.stringify(jsonData) },'#scatName',
			(err,data) => {
				if (data.output == 0) {
					Scat._showAlert('Service Catalog component saved');
				}
		});
	},
	
	/**
	 * Load selected Service Catalog
	 */
	load: () => {
	
		var id = $('#scats').children('option:selected').val();
		Scat._makeRequest('POST','getservicecatalogs', {  id: id },'#scats',
			(err,data) => {
				if (err) return;
				
				// Set Service Catalog ID
				Scat._id = data.id;
				
				// Set Service Catalog Content
				$('#scatName').val( data.name );
				$('#scatType').val( data.type );
				$('#scatRevision').html( data.last_revision.replace(/\//g,'-'));
				$('#scatCost').val( data.cost );
				$('#scatTimeCut').val( data.selectedTimecut );
				$('#scatResolutionTime').val( data.resolutionTime );
				$('#scatTreatmentTime').val( data.answerTime );
				$('#scatSeverity').val( data.selectedSeverity );
				$('#scatFinancialLoss').val( data.imageInjury );
				$('#scatComplexity').val( data.complexity );
				$('#scatVIPUsers').val( data.vipUsers );
				$('#scatAffectedUsers').val( data.selectedAffectedUsers );
				$('#scatDependencyRelationship').val( data.dependencyRelationship );
				$('#scatDescription').val( data.description );
				$('#scatDependentBusinessProcesses').val( data.dependentBusinessProcesses );
				$('#scatServiceDeliverable').val( data.serviceDeliverable );
				$('#scatTxtUsers').val( data.txtUser );
				$('#scatTxtKeyUsers').val( data.txtKeyUser );
				$('#scatTxtBusinessAccountable').val( data.txtBusinessAccountable );
				$('#scatTxtTechnicalAccountable').val( data.txtTechnicalAccountable );
				$('#scatInvolvedApps').val( data.involvedApplications );

				// Set seasonality
				$.each( data.sazDays, (i,v) => {
					$('#sazDay_' + i).prop('checked', v == 1);
				});
			
				// Set custom fields
				$('#scatCustomAttributes').DataTable().clear().draw();
				$.each( data.custom_fields, (i,v) => {
					$('#scatCustomAttributes').DataTable().row.add([v.cf_name,v.cf_value,null]).draw();
				});
				
				// Set selected contacts
				Scat._selectContacts( data.selectedUser,                 '#scatUsers' );
				Scat._selectContacts( data.selectedKeyUser,              '#scatKeyUsers' );
				Scat._selectContacts( data.selectedBusinessAccountable,  '#scatBusinessAccountable' );
				Scat._selectContacts( data.selectedTechnicalAccountable, '#scatTechnicalAccountable' );
				var resp = data.responsable.shift();
				$('#scatResponsable').val( (resp.type == 10 ? 'g' : 'c') + '_' + resp.value );
				
				// Populate levels
				Scat._scatContent = new Array();
				$.each( data.levels, (i,v) => {
					
					//
					// Single object
					//
					if (v.length == 1) {
						
						//
						// Host/Service
						//
						if (v[0].service_id) {
							Scat._addScatContent([
								v[0].host_id + "!" + v[0].service_id,
								v[0].host_id + "!" + v[0].service_id,
								v[0].host_name + " - " + v[0].service_description,
								v[0].service_description
							]);
							
						//
						// Service Catalog
						//
						} else {
							Scat._addScatContent([
								v[0].host_id,
								v[0].host_id,
								"Service Catalog: " + v[0].host_name,
								v[0].host_name
							]);
						}
						
					//
					// Multi objects
					//
					} else {
						
						var arr = [ [], [], [], [] ];
						$.each(v, (idx, obj) => {
							//
							// Host/Service
							//
							if (obj.service_id) {
								arr[0].push(obj.host_id + "!" + obj.service_id);
								arr[1].push(obj.host_id + "!" + obj.service_id);
								arr[2].push(obj.host_name + " - " + obj.service_description);
								arr[3].push(obj.service_description);

							//
							// Service Catalog
							//
							} else {
								arr[0].push(obj.host_id);
								arr[1].push(obj.host_id);
								arr[2].push("Service Catalog: " + obj.host_name);
								arr[3].push(obj.host_name);
							}
						});
						Scat._scatContent.push(arr);
					}
				});
				Scat._applyScatContent();
				
				// Load SLA
				Scat.loadSla();
				
		});
		
		// Fix scat remove label
		$('#scatRemoveLabel').html('Removing ' + $('#scats').children('option:selected').text());
		
	},
	
	/**
	 * Show modal to remove scat
	 * 
	 * @returns void
	 */
	removeModal: () => {
		if (Scat._id !== null)
			$('#scatRemove').modal();
	},
	
	/**
	 * Remove Service Catalog
	 */
	remove: () => {
		
		if (Scat._id !== null) {
			Scat._makeRequest('POST','removescat', { id: Scat._id },'#scats',
				(err,data) => {
					$('#scatRemove').modal('hide');
					Scat._showAlert('Service Catalog ' + $('#scats').children('option:selected').text() + ' removed');
					Scat._id = null;
					Scat._scatContent = [];
					Scat._applyScatContent();
					$('#scats').children('option:selected').remove();
					$('#scats').val(0);
					$('#scatCustomAttributes').DataTable().clear().draw();
					$.each(['scatUsers','scatKeyUsers','scatBusinessAccountable','scatTechnicalAccountable'], (i,v) => {
						$('#' + v).DataTable().rows().deselect();
					});
				}
			);
		}
	},
	
	/**
	 * Load saved SLA
	 * 
	 * @returns void
	 */
	loadSla: () => {
		
		if (Scat._id === null) return;
		
		var year    = $('#scatSlaYear').children('option:selected').text();
		var timecut = $('#scatSlaTimeCut').val();
		
		Scat._makeRequest('POST','getslas', { json: JSON.stringify({
				children:       true,
				selected_itens: [{ value: Scat._id }],
				year:           year,
				timecut_id:     timecut,
				object_type:    4
			})},
			'#scats',
			(err,data) => {
				Scat.clearSlaGrid();
				if (!data.length) return;
				$.each( data, (i,v) => {
					$('.sla-month:eq(' + i + ')').html(v.value);
				});
				$('.sla-warning').html(data[0].warning);
			}
		);
		
	},

	/**
	 * Initialize DataTable
	 * 
	 * @returns void
	 */
  initializeDataTable: () => {
    $('#scatObjectContent').DataTable({
      paging:    false,
      searching: false,
      info:      false,
      select:    true,
      data:      [],
      order:     [[1,'asc']],
      scrollY:   200,
      language:  { emptyTable: "No content available!" },
      columns: [
        { visible: false },
        { title: "Elements" }
      ],
      "drawCallback": () => {
        // Enable double click
        $('#scatObjectContent').unbind('dblclick').on('dblclick','tr',(e) => {
          e.stopPropagation();
          var row = $( e.target ).parent();
          $('#scatObjectContent').DataTable().row( row ).select();
          Scat.addItem();
        });
      }
    });
  },

  /**
   * Populate DataTable
   * 
   * @returns void
  */
  populateDataTable: (arr) => {
    var dt = $('#scatObjectContent').DataTable();
    dt.clear();
    dt.rows.add( arr );
    dt.draw();
  }
	
};

$().ready( Scat.init );
